@@ -808,8 +808,8 @@ namespace Tomboy
 
 			// Remove tags now, since a note with no tags has
 			// no "tags" element in the XML
-			foreach (Tag tag in Tags)
-			RemoveTag (tag);
+			List<Tag> newTags = new List<Tag> ();
+			DateTime date;
 
 			while (xml.Read ()) {
 				switch (xml.NodeType) {
@@ -822,27 +822,35 @@ namespace Tomboy
 						XmlContent = xml.ReadInnerXml ();
 						break;
 					case "last-change-date":
-						data.Data.ChangeDate =
-						        XmlConvert.ToDateTime (xml.ReadString (), NoteArchiver.DATE_TIME_FORMAT);
+						if (DateTime.TryParse (xml.ReadString (), out date))
+							data.Data.ChangeDate = date;
+						else
+							data.Data.ChangeDate = DateTime.Now;
 						break;
 					case "last-metadata-change-date":
-						data.Data.MetadataChangeDate =
-						        XmlConvert.ToDateTime (xml.ReadString (), NoteArchiver.DATE_TIME_FORMAT);
+						if (DateTime.TryParse (xml.ReadString (), out date))
+							data.Data.MetadataChangeDate = date;
+						else
+							data.Data.MetadataChangeDate = DateTime.Now;
 						break;
 					case "create-date":
-						data.Data.CreateDate =
-						        XmlConvert.ToDateTime (xml.ReadString (), NoteArchiver.DATE_TIME_FORMAT);
+						if (DateTime.TryParse (xml.ReadString (), out date))
+							data.Data.CreateDate = date;
+						else
+							data.Data.CreateDate = DateTime.Now;
 						break;
 					case "tags":
 						XmlDocument doc = new XmlDocument ();
 						List<string> tag_strings = ParseTags (doc.ReadNode (xml.ReadSubtree ()));
 						foreach (string tag_str in tag_strings) {
 							Tag tag = TagManager.GetOrCreateTag (tag_str);
-							AddTag (tag);
+							newTags.Add (tag);
 						}
 						break;
 					case "open-on-startup":
-						IsOpenOnStartup = bool.Parse (xml.ReadString ());
+						bool isStartup;
+						if (bool.TryParse (xml.ReadString (), out isStartup))
+							IsOpenOnStartup = isStartup;
 						break;
 					}
 					break;
@@ -851,6 +859,12 @@ namespace Tomboy
 
 			xml.Close ();
 
+			foreach (Tag oldTag in Tags)
+				if (!newTags.Contains (oldTag))
+				    RemoveTag (oldTag);
+			foreach (Tag newTag in newTags)
+				AddTag (newTag);
+
 			// Allow method caller to specify ChangeType (mostly needed by sync)
 			QueueSave (changeType);
 		}
@@ -937,19 +951,10 @@ namespace Tomboy
 		{
 			get {
 				if (tag_table == null) {
-					#if FIXED_GTKSPELL
 					// NOTE: Sharing the same TagTable means
 					// that formatting is duplicated between
 					// buffers.
 					tag_table = NoteTagTable.Instance;
-					#else
-					// NOTE: GtkSpell chokes on shared
-					// TagTables because it blindly tries to
-					// create a new "gtkspell-misspelling"
-					// tag, which fails if one already
-					// exists in the table.
-					tag_table = new NoteTagTable ();
-					#endif
 				}
 				return tag_table;
 			}
@@ -1112,6 +1117,8 @@ namespace Tomboy
 	{
 		public const string CURRENT_VERSION = "0.3";
 
+		// NOTE: If this changes from a standard format, make sure to update
+		//       XML parsing to have a DateTime.TryParseExact
 		public const string DATE_TIME_FORMAT = "yyyy-MM-ddTHH:mm:ss.fffffffzzz";
 
 		static NoteArchiver instance = null;
@@ -1154,6 +1161,8 @@ namespace Tomboy
 			                                        System.Text.Encoding.UTF8);
 			XmlTextReader xml = new XmlTextReader (reader);
 			xml.Namespaces = false;
+			DateTime date;
+			int num;
 
 			while (xml.Read ()) {
 				switch (xml.NodeType) {
@@ -1171,31 +1180,42 @@ namespace Tomboy
 						note.Text = xml.ReadInnerXml ();
 						break;
 					case "last-change-date":
-						note.ChangeDate =
-						        XmlConvert.ToDateTime (xml.ReadString (), DATE_TIME_FORMAT);
+						if (DateTime.TryParse (xml.ReadString (), out date))
+							note.ChangeDate = date;
+						else
+							note.ChangeDate = DateTime.Now;
 						break;
 					case "last-metadata-change-date":
-						note.MetadataChangeDate =
-						        XmlConvert.ToDateTime (xml.ReadString (), DATE_TIME_FORMAT);
+						if (DateTime.TryParse (xml.ReadString (), out date))
+							note.MetadataChangeDate = date;
+						else
+							note.MetadataChangeDate = DateTime.Now;
 						break;
 					case "create-date":
-						note.CreateDate =
-						        XmlConvert.ToDateTime (xml.ReadString (), DATE_TIME_FORMAT);
+						if (DateTime.TryParse (xml.ReadString (), out date))
+							note.CreateDate = date;
+						else
+							note.CreateDate = DateTime.Now;
 						break;
 					case "cursor-position":
-						note.CursorPosition = int.Parse (xml.ReadString ());
+						if (int.TryParse (xml.ReadString (), out num))
+							note.CursorPosition = num;
 						break;
 					case "width":
-						note.Width = int.Parse (xml.ReadString ());
+						if (int.TryParse (xml.ReadString (), out num))
+							note.Width = num;
 						break;
 					case "height":
-						note.Height = int.Parse (xml.ReadString ());
+						if (int.TryParse (xml.ReadString (), out num))
+							note.Height = num;
 						break;
 					case "x":
-						note.X = int.Parse (xml.ReadString ());
+						if (int.TryParse (xml.ReadString (), out num))
+							note.X = num;
 						break;
 					case "y":
-						note.Y = int.Parse (xml.ReadString ());
+						if (int.TryParse (xml.ReadString (), out num))
+							note.Y = num;
 						break;
 					case "tags":
 						XmlDocument doc = new XmlDocument ();
@@ -1206,7 +1226,9 @@ namespace Tomboy
 						}
 						break;
 					case "open-on-startup":
-						note.IsOpenOnStartup = bool.Parse (xml.ReadString ());
+						bool isStartup;
+						if (bool.TryParse (xml.ReadString (), out isStartup))
+							note.IsOpenOnStartup = isStartup;
 						break;
 					}
 					break;
@@ -1426,7 +1448,10 @@ namespace Tomboy
 			if (notes.Count == 1)
 				message = Catalog.GetString ("Really delete this note?");
 			else
-				message = Catalog.GetString ("Really delete these notes?");
+				message = string.Format (Catalog.GetPluralString (
+					"Really delete this {0} note?",
+					"Really delete these {0} notes?",
+					notes.Count), notes.Count);
 			
 			HIGMessageDialog dialog =
 			        new HIGMessageDialog (
@@ -1463,6 +1488,16 @@ namespace Tomboy
 		
 		public static void ShowIOErrorDialog (Gtk.Window parent)
 		{
+			string errorMsg = Catalog.GetString ("An error occurred while saving your notes. " +
+			                                     "Please check that you have sufficient disk " +
+			                                     "space, and that you have appropriate rights " +
+			                                     "on {0}. Error details can be found in " +
+			                                     "{0}.");
+			string logPath = System.IO.Path.Combine (Services.NativeApplication.LogDirectory,
+			                                         "tomboy.log");
+			errorMsg = String.Format (errorMsg,
+			                          Services.NativeApplication.DataDirectory,
+			                          logPath);
 			HIGMessageDialog dialog =
 				new HIGMessageDialog (
 				                      parent,
@@ -1470,11 +1505,7 @@ namespace Tomboy
 				                      Gtk.MessageType.Error,
 				                      Gtk.ButtonsType.Ok,
 				                      Catalog.GetString ("Error saving note data."),
-				                      Catalog.GetString ("An error occurred while saving your notes. " +
-				                                         "Please check that you have sufficient disk " +
-				                                         "space, and that you have appropriate rights " +
-				                                         "on ~/.tomboy. Error details can be found in " +
-				                                         "~/.tomboy.log."));
+				                      errorMsg);
 			dialog.Run ();
 			dialog.Destroy ();
 		}
