@@ -1,6 +1,5 @@
 
 using System;
-using System.Collections;
 using System.Collections.Generic;
 using System.IO;
 using System.Xml;
@@ -14,7 +13,12 @@ namespace Tomboy
 	public class NoteBuffer : Gtk.TextBuffer
 	{
 		UndoManager undo_manager;
-		char[] indent_bullets = {'\u2022', '\u2218', '\u2023'};
+		char[] indent_bullets = {
+			'\u2022',
+#if !MAC
+			'\u2218', // Not available on Mac, need to pick something else
+#endif
+			'\u2023'};
 
 		// GODDAMN Gtk.TextBuffer. I hate you. Hate Hate Hate.
 		struct WidgetInsertData
@@ -30,7 +34,7 @@ namespace Tomboy
 		// HATE.
 
 		// list of Gtk.TextTags to apply on insert
-		ArrayList active_tags;
+		List<Gtk.TextTag> active_tags;
 
 		// The note that owns this buffer
 		private Note note;
@@ -47,7 +51,7 @@ namespace Tomboy
 		public NoteBuffer (Gtk.TextTagTable tags, Note note)
 : base (tags)
 		{
-			active_tags = new ArrayList ();
+			active_tags = new List<Gtk.TextTag> ();
 			undo_manager = new UndoManager (this);
 
 			InsertText += TextInsertedEvent;
@@ -131,6 +135,8 @@ namespace Tomboy
 		/// </summary>
 		public DynamicNoteTag GetDynamicTag (string tag_name, Gtk.TextIter iter)
 		{
+			// TODO: Is this variables used, or do we just need to
+			// access iter.Tags to work around a bug?
 			Gtk.TextTag [] tags = iter.Tags;
 			foreach (Gtk.TextTag tag in iter.Tags) {
 				DynamicNoteTag dynamic_tag = tag as DynamicNoteTag;
@@ -276,7 +282,7 @@ namespace Tomboy
 			}
 		}
 
-		public bool AddNewline()
+		public bool AddNewline(bool soft_break)
 		{
 			if (!CanMakeBulletedList() || !EnableAutoBulletedLists)
 				return false;
@@ -286,16 +292,34 @@ namespace Tomboy
 			iter.LineOffset = 0;
 
 			DepthNoteTag prev_depth = FindDepthTag (iter);
+			
+			Gtk.TextIter insert = GetIterAtMark (insert_mark);
+ 
+			// Insert a LINE SEPARATOR character which allows us
+			// to have multiple lines in a single bullet point
+			if (prev_depth != null && soft_break) {
+				bool at_end_of_line = insert.EndsLine ();
+				Insert (ref insert, "\u2028");
+				
+				// Hack so that the user sees that what they type
+				// next will appear on a new line, otherwise the
+				// cursor stays at the end of the previous line.
+				if (at_end_of_line) {
+					Insert (ref insert, " ");
+					Gtk.TextIter bound = insert;
+					bound.BackwardChar ();
+					MoveMark (SelectionBound, bound);
+				}
+				
+				return true;			
 
 			// If the previous line has a bullet point on it we add a bullet
 			// to the new line, unless the previous line was blank (apart from
 			// the bullet), in which case we clear the bullet/indent from the
 			// previous line.
-			if (prev_depth != null) {
+			} else if (prev_depth != null) {
 				iter.ForwardChar ();
 
-				Gtk.TextIter insert = GetIterAtMark (insert_mark);
-
 				// See if the line was left contentless and remove the bullet
 				// if so.
 				if (iter.EndsLine () || insert.LineOffset < 3 ) {
@@ -314,8 +338,16 @@ namespace Tomboy
 					iter = GetIterAtMark (insert_mark);
 					Insert (ref iter, "\n");
 				} else {
-					Undoer.FreezeUndo ();
 					iter = GetIterAtMark (insert_mark);
+					Gtk.TextIter prev = iter;
+					prev.BackwardChar ();
+					
+					// Remove soft breaks
+					if (prev.Char == "\u2028") {
+						Delete (ref prev, ref iter);
+					}
+					
+					Undoer.FreezeUndo ();
 					int offset = iter.Offset;
 					Insert (ref iter, "\n");
 
@@ -337,15 +369,18 @@ namespace Tomboy
 
 				return true;
 			}
-			// Replace lines starting with '*' or '-' with bullets
-			else if (iter.Char.Equals ("*") || iter.Char.Equals ("-")) {
+			// Replace lines starting with any numbers of leading spaces 
+			// followed by '*' or '-' and then by a space with bullets
+			else if (LineNeedsBullet(iter)) {
 				Gtk.TextIter start = GetIterAtLineOffset (iter.Line, 0);
-				Gtk.TextIter end = GetIterAtLineOffset (iter.Line, 1);
+				Gtk.TextIter end = GetIterAtLineOffset (iter.Line, 0);
 
-				// Remove the '*' character and any leading white space
-				if (end.Char == " ")
+				// Remove any leading white space
+				while (end.Char == " ")
 					end.ForwardChar();
-
+				// Remove the '*' or '-' character and the space after
+				end.ForwardChars(2);
+				
 				// Set the direction of the bullet to be the same as
 				// the first character after the '*' or '-'
 				Pango.Direction direction = Pango.Direction.Ltr;
@@ -380,6 +415,29 @@ namespace Tomboy
 			return false;
 		}
 
+		// Returns true if line starts with any numbers of leading spaces
+		// followed by '*' or '-' and then by a space
+		private bool LineNeedsBullet(Gtk.TextIter iter)
+		{
+			while (!iter.EndsLine ()) {
+				switch (iter.Char) {
+				case " ":
+					iter.ForwardChar ();
+					break;
+				case "*":
+				case "-":
+					if (GetIterAtLineOffset(iter.Line, iter.LineOffset + 1).Char.Equals(" ")) {
+						return true;
+					} else {
+						return false;
+					}
+				default:
+					return false;
+				}
+			}
+			return false;
+		}
+		
 		// Returns true if the depth of the line was increased
 		public bool AddTab ()
 		{
@@ -489,6 +547,18 @@ namespace Tomboy
 				if (depth != null || prev_depth != null) {
 					DecreaseDepth (ref start);
 					return true;
+				} else {
+					// See if the cursor is before a soft line break
+					// and remove it if it is. Otherwise you have to
+					// press backspace twice before  it will delete
+					// the previous visible character.
+					prev = start;
+					prev.BackwardChars (2);
+					if (prev.Char == "\u2028") {
+						Gtk.TextIter end_break = prev;
+						end_break.ForwardChar ();
+						Delete (ref prev, ref end_break);
+					}
 				}
 			}
 
@@ -613,6 +683,10 @@ namespace Tomboy
 		public bool RunWidgetQueue ()
 		{
 			foreach (WidgetInsertData data in widgetQueue) {
+				// HACK: This is a quick fix for bug #486551
+				if (data.position == null)
+					continue;
+				
 				NoteBuffer buffer = data.buffer as NoteBuffer;
 				Gtk.TextIter iter = buffer.GetIterAtMark (data.position);
 				Gtk.TextMark location = data.position;
@@ -744,7 +818,6 @@ namespace Tomboy
 
 			bool toggle_on = true;
 			if (FindDepthTag (start) != null) {
-				Gtk.TextIter bullet_end = GetIterAtLineOffset (start.Line, 2);
 				toggle_on = false;
 			}
 
@@ -1068,7 +1141,16 @@ namespace Tomboy
 			Serialize (buffer, start, end, xml);
 
 			xml.Close ();
-			return stream.ToString ();
+			string serializedBuffer = stream.ToString ();
+			
+			// We cannot use newer XmlWriter with XmlWriterSettings
+			// to control the newline character, because XmlWriter
+			// doesn't like elements with ":" in the name.  The
+			// point here is to write these files identically on
+			// all platforms, to make synchronization work better.
+			if (Environment.NewLine != "\n")
+				serializedBuffer = serializedBuffer.Replace (Environment.NewLine, "\n");
+			return serializedBuffer;
 		}
 
 		static void WriteTag (Gtk.TextTag tag, XmlTextWriter xml, bool start)
@@ -1098,9 +1180,9 @@ namespace Tomboy
 		                              Gtk.TextIter   end,
 		                              XmlTextWriter  xml)
 		{
-			Stack tag_stack = new Stack ();
-			Stack replay_stack = new Stack ();
-			Stack continue_stack = new Stack ();
+			Stack<Gtk.TextTag> tag_stack = new Stack<Gtk.TextTag> ();
+			Stack<Gtk.TextTag> replay_stack = new Stack<Gtk.TextTag> ();
+			Stack<Gtk.TextTag> continue_stack = new Stack<Gtk.TextTag> ();
 
 			Gtk.TextIter iter = start;
 			Gtk.TextIter next_iter = start;
@@ -1122,8 +1204,6 @@ namespace Tomboy
 			}
 
 			while (!iter.Equal (end) && iter.Char != null) {
-				bool new_list = false;
-
 				DepthNoteTag depth_tag = ((NoteBuffer)buffer).FindDepthTag (iter);
 
 				// If we are at a character with a depth tag we are at the
@@ -1168,7 +1248,6 @@ namespace Tomboy
 							xml.WriteStartElement (null, "list-item", null);
 							xml.WriteStartElement (null, "list", null);
 						}
-						new_list = true;
 					}
 
 					prev_depth = depth_tag.Depth;
@@ -1193,7 +1272,7 @@ namespace Tomboy
 				while (continue_stack.Count > 0 &&
 				                ((depth_tag == null && iter.StartsLine ()) || iter.LineOffset == 1))
 				{
-					Gtk.TextTag continue_tag = (Gtk.TextTag) continue_stack.Pop();
+					Gtk.TextTag continue_tag = continue_stack.Pop();
 
 					if (!TagEndsHere (continue_tag, iter, next_iter)
 					                && iter.HasTag (continue_tag))
@@ -1212,6 +1291,9 @@ namespace Tomboy
 						if (serialize != null)
 							xml.WriteRaw (serialize);
 					}
+				// Line Separator character
+				} else if (iter.Char == "\u2028") {
+					xml.WriteCharEntity ('\u2028');
 				} else if (depth_tag == null) {
 					xml.WriteString (iter.Char);
 				}
@@ -1232,7 +1314,7 @@ namespace Tomboy
 				{
 					// Close all tags in the tag_stack
 					while (tag_stack.Count > 0) {
-						Gtk.TextTag existing_tag = tag_stack.Pop () as Gtk.TextTag;
+						Gtk.TextTag existing_tag = tag_stack.Pop ();
 
 						// Any tags which continue across the indented
 						// line are added to the continue_stack to be
@@ -1249,7 +1331,7 @@ namespace Tomboy
 						                NoteTagTable.TagIsSerializable(tag) && !(tag is DepthNoteTag))
 						{
 							while (tag_stack.Count > 0) {
-								Gtk.TextTag existing_tag = tag_stack.Pop () as Gtk.TextTag;
+								Gtk.TextTag existing_tag = tag_stack.Pop ();
 
 								if (!TagEndsHere (existing_tag, iter, next_iter)) {
 									replay_stack.Push (existing_tag);
@@ -1263,7 +1345,7 @@ namespace Tomboy
 							// overlapped with the ended
 							// tag...
 							while (replay_stack.Count > 0) {
-								Gtk.TextTag replay_tag = replay_stack.Pop () as Gtk.TextTag;
+								Gtk.TextTag replay_tag = replay_stack.Pop ();
 								tag_stack.Push (replay_tag);
 
 								WriteTag (replay_tag, xml, true);
@@ -1299,7 +1381,7 @@ namespace Tomboy
 
 			// Empty any trailing tags left in tag_stack..
 			while (tag_stack.Count > 0) {
-				Gtk.TextTag tail_tag = (Gtk.TextTag) tag_stack.Pop ();
+				Gtk.TextTag tail_tag = tag_stack.Pop ();
 				WriteTag (tail_tag, xml, false);
 			}
 
@@ -1333,7 +1415,7 @@ namespace Tomboy
 		                                XmlTextReader  xml)
 		{
 			int offset = start.Offset;
-			Stack stack = new Stack ();
+			Stack<TagStart> stack = new Stack<TagStart> ();
 			TagStart tag_start;
 
 			NoteTagTable note_table = buffer.TagTable as NoteTagTable;
@@ -1342,7 +1424,7 @@ namespace Tomboy
 
 			// A stack of boolean values which mark if a
 			// list-item contains content other than another list
-			Stack list_stack = new Stack();
+			Stack<bool> list_stack = new Stack<bool> ();
 
 			while (xml.Read ()) {
 				switch (xml.NodeType) {
@@ -1408,7 +1490,7 @@ namespace Tomboy
 						break;
 					}
 
-					tag_start = (TagStart) stack.Pop ();
+					tag_start = stack.Pop ();
 					if (tag_start.Tag == null)
 						break;
 
@@ -1425,7 +1507,7 @@ namespace Tomboy
 					// had content.
 					DepthNoteTag depth_tag = tag_start.Tag as DepthNoteTag;
 
-					if (depth_tag != null && (bool) list_stack.Pop ()) {
+					if (depth_tag != null && list_stack.Pop ()) {
 						((NoteBuffer) buffer).InsertBullet (ref apply_start,
 						                                    depth_tag.Depth,
 						                                    depth_tag.Direction);
