@@ -1,6 +1,6 @@
 
 using System;
-using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 using System.Runtime.InteropServices;
 using System.Xml;
@@ -17,6 +17,7 @@ namespace Tomboy
 		string element_name;
 		Gtk.TextMark widgetLocation;
 		Gtk.Widget widget;
+		bool allow_middle_activate = false;
 
 		[Flags]
 		enum TagFlags {
@@ -198,6 +199,18 @@ namespace Tomboy
 			case Gdk.EventType.ButtonPress:
 				Gdk.EventButton button_ev = new Gdk.EventButton (ev.Handle);
 
+				// Do not insert selected text when activating links with
+				// middle mouse button
+				if (button_ev.Button == 2) {
+					allow_middle_activate = true;
+					return true;
+				}
+
+				return false;
+
+			case Gdk.EventType.ButtonRelease:
+				button_ev = new Gdk.EventButton (ev.Handle);
+
 				if (button_ev.Button != 1 && button_ev.Button != 2)
 					return false;
 
@@ -206,15 +219,27 @@ namespace Tomboy
 				                              Gdk.ModifierType.ControlMask)) != 0)
 					return false;
 
+				// Prevent activation when selecting links with the mouse
+				if (editor.Buffer.HasSelection)
+					return false;
+
+				// Don't activate if the link has just been pasted with the
+				// middle mouse button (no preceding ButtonPress event)
+				if (button_ev.Button == 2 && !allow_middle_activate)
+					return false;
+				else
+					allow_middle_activate = false;
+
 				GetExtents (iter, out start, out end);
 				bool success = OnActivate (editor, start, end);
 
+				// Hide note if link is activated with middle mouse button
 				if (success && button_ev.Button == 2) {
 					Gtk.Widget widget = (Gtk.Widget) sender;
 					widget.Toplevel.Hide ();
 				}
 
-				return success;
+				return false;
 
 			case Gdk.EventType.KeyPress:
 				Gdk.EventKey key_ev = new Gdk.EventKey (ev.Handle);
@@ -347,18 +372,18 @@ namespace Tomboy
 
 	public class DynamicNoteTag : NoteTag
 	{
-		Hashtable attributes;
+		Dictionary<string, string> attributes;
 
 		public DynamicNoteTag ()
 : base()
 		{
 		}
 
-		public Hashtable Attributes
+		public IDictionary<string, string> Attributes
 		{
 			get {
 				if (attributes == null)
-					attributes = new Hashtable ();
+					attributes = new Dictionary<string, string> ();
 				return attributes;
 			}
 		}
@@ -370,7 +395,7 @@ namespace Tomboy
 
 				if (start && attributes != null) {
 					foreach (string key in attributes.Keys) {
-						string val = (string) attributes [key];
+						string val = attributes [key];
 						xml.WriteAttributeString (null, key, null, val);
 					}
 				}
@@ -423,7 +448,7 @@ namespace Tomboy
 			}
 		}
 
-		public Pango.Direction Direction
+		public new Pango.Direction Direction
 		{
 			get{
 				return direction;
@@ -460,8 +485,8 @@ namespace Tomboy
 	public class NoteTagTable : Gtk.TextTagTable
 	{
 		static NoteTagTable instance;
-		Hashtable tag_types;
-		ArrayList added_tags;
+		Dictionary<string, Type> tag_types;
+		List<Gtk.TextTag> added_tags;
 
 		public static NoteTagTable Instance
 		{
@@ -475,12 +500,16 @@ namespace Tomboy
 		public NoteTagTable ()
 : base ()
 		{
-			tag_types = new Hashtable ();
-			added_tags = new ArrayList ();
+			tag_types = new Dictionary<string, Type> ();
+			added_tags = new List<Gtk.TextTag> ();
 
 			InitCommonTags ();
 		}
 
+		public NoteTag UrlTag { get; private set; }
+		public NoteTag LinkTag { get; private set; }
+		public NoteTag BrokenLinkTag { get; private set; }
+
 		void InitCommonTags ()
 		{
 			NoteTag tag;
@@ -589,6 +618,7 @@ namespace Tomboy
 			        ContrastPaletteColor.Grey;
 			tag.CanActivate = true;
 			Add (tag);
+			BrokenLinkTag = tag;
 
 			tag = new NoteTag ("link:internal");
 			tag.Underline = Pango.Underline.Single;
@@ -596,6 +626,7 @@ namespace Tomboy
 			        ContrastPaletteColor.Blue;
 			tag.CanActivate = true;
 			Add (tag);
+			LinkTag = tag;
 
 			tag = new NoteTag ("link:url");
 			tag.Underline = Pango.Underline.Single;
@@ -603,6 +634,7 @@ namespace Tomboy
 			        ContrastPaletteColor.Blue;
 			tag.CanActivate = true;
 			Add (tag);
+			UrlTag = tag;
 		}
 
 		public static bool TagIsSerializable (Gtk.TextTag tag)
@@ -673,8 +705,8 @@ namespace Tomboy
 
 		public DynamicNoteTag CreateDynamicTag (string tag_name)
 		{
-			Type tag_type = tag_types [tag_name] as Type;
-			if (tag_type == null)
+			Type tag_type;
+			if (!tag_types.TryGetValue (tag_name, out tag_type))
 				return null;
 
 			DynamicNoteTag tag = (DynamicNoteTag) Activator.CreateInstance(tag_type);
@@ -693,7 +725,11 @@ namespace Tomboy
 
 		public bool IsDynamicTagRegistered (string tag_name)
 		{
-			return tag_types [tag_name] != null;
+			Type type;
+			if (tag_types.TryGetValue (tag_name, out type) &&
+			    type != null)
+				return true;
+			return false;
 		}
 
 		protected override void OnTagAdded (Gtk.TextTag tag)
