@@ -61,7 +61,6 @@ namespace Tomboy
 			Resizable = true;
 			Title = Catalog.GetString ("Tomboy Preferences");
 
-			VBox.Spacing = 5;
 			ActionArea.Layout = Gtk.ButtonBoxStyle.End;
 
 			addin_prefs_dialogs =
@@ -73,7 +72,6 @@ namespace Tomboy
 
 			Gtk.Notebook notebook = new Gtk.Notebook ();
 			notebook.TabPos = Gtk.PositionType.Top;
-			notebook.BorderWidth = 5;
 			notebook.Show ();
 
 			notebook.AppendPage (MakeEditingPane (),
@@ -448,9 +446,9 @@ namespace Tomboy
 
 			syncAddinPrefsWidget.Show ();
 			syncAddinPrefsContainer = new Gtk.VBox (false, 0);
-			syncAddinPrefsContainer.PackStart (syncAddinPrefsWidget, true, true, 0);
+			syncAddinPrefsContainer.PackStart (syncAddinPrefsWidget, false, false, 0);
 			syncAddinPrefsContainer.Show ();
-			vbox.PackStart (syncAddinPrefsContainer, true, true, 0);
+			vbox.PackStart (syncAddinPrefsContainer, true, true, 10);
 
 			Gtk.HButtonBox bbox = new Gtk.HButtonBox ();
 			bbox.Spacing = 4;
@@ -537,7 +535,23 @@ namespace Tomboy
 			sw.ShadowType = Gtk.ShadowType.In;
 			sw.Add (tree);
 			sw.Show ();
-			hbox.PackStart (sw, true, true, 0);
+			Gtk.LinkButton get_more_link =
+				new Gtk.LinkButton ("http://live.gnome.org/Tomboy/PluginList",
+				                    Catalog.GetString ("Get More Add-Ins..."));
+			get_more_link.Clicked += delegate(object sender, EventArgs args) {
+				string uri = ((Gtk.LinkButton) sender).Uri;
+				try {
+					Services.NativeApplication.OpenUrl (uri, Screen);
+				} catch (Exception e) {
+					GuiUtils.ShowOpeningLocationError (this, uri, e.Message);
+				}
+			};
+			get_more_link.Show ();
+			Gtk.VBox tree_box = new Gtk.VBox (false, 0);
+			tree_box.Add (sw);
+			tree_box.PackEnd (get_more_link, false, false, 5);
+			tree_box.Show ();
+			hbox.PackStart (tree_box, true, true, 0);
 
 			// Action Buttons (right of TreeView)
 			Gtk.VButtonBox button_box = new Gtk.VButtonBox ();
@@ -1021,7 +1035,7 @@ namespace Tomboy
 					}
 
 					syncAddinPrefsWidget.Show ();
-					syncAddinPrefsContainer.PackStart (syncAddinPrefsWidget, true, true, 0);
+					syncAddinPrefsContainer.PackStart (syncAddinPrefsWidget, false, false, 0);
 
 					resetSyncAddinButton.Sensitive = false;
 					saveSyncAddinButton.Sensitive = true;
@@ -1091,7 +1105,7 @@ namespace Tomboy
 			        Preferences.SYNC_CONFIGURED_CONFLICT_BEHAVIOR,
 			        Preferences.GetDefault (Preferences.SYNC_CONFIGURED_CONFLICT_BEHAVIOR));
 
-			NukeSyncClientManifest ();
+			SyncManager.ResetClient ();
 
 			syncAddinCombo.Sensitive = true;
 			resetSyncAddinButton.Sensitive = false;
@@ -1100,26 +1114,6 @@ namespace Tomboy
 				syncAddinPrefsWidget.Sensitive = true;
 		}
 
-		// TODO: This class shouldn't know about the manifest.
-		//       Abstract this out somehow (ie, SyncManager.ResetClient()
-		//       or something like that).
-		void NukeSyncClientManifest ()
-		{
-			// Nuke ~/.tomboy/manifest.xml
-			string clientManifestPath = System.IO.Path.Combine (
-			                                    Tomboy.DefaultNoteManager.NoteDirectoryPath,
-			                                    "manifest.xml");
-			if (System.IO.File.Exists (clientManifestPath) == true) {
-				try {
-					System.IO.File.Delete (clientManifestPath);
-				} catch (Exception e) {
-					Logger.Debug ("Error deleting \"{0}\" during reset: {1}",
-					              clientManifestPath,
-					              e.Message);
-				}
-			}
-		}
-
 		/// <summary>
 		/// Attempt to save/test the connection to the sync addin.
 		/// </summary>
@@ -1155,7 +1149,7 @@ namespace Tomboy
 				resetSyncAddinButton.Sensitive = true;
 				saveSyncAddinButton.Sensitive = false;
 
-				NukeSyncClientManifest ();
+				SyncManager.ResetClient ();
 
 				// Give the user a visual letting them know that connecting
 				// was successful.
@@ -1189,11 +1183,15 @@ namespace Tomboy
 
 				// Give the user a visual letting them know that connecting
 				// was successful.
-				if (errorMsg == null)
+				if (errorMsg == null) {
 					errorMsg = Catalog.GetString ("Sorry, but something went wrong.  " +
 					                              "Please check your information and " +
-					                              "try again.  The ~/.tomboy.log might " +
+					                              "try again.  The {0} might " +
 					                              "be useful too.");
+					string logPath = System.IO.Path.Combine (Services.NativeApplication.LogDirectory,
+					                                         "tomboy.log");
+					errorMsg = String.Format (errorMsg, logPath);
+				}
 				dialog =
 				        new HIGMessageDialog (this,
 				                              Gtk.DialogFlags.Modal,
